@@ -25,8 +25,8 @@
 #include <getopt.h>
 #include <dirent.h>
 #include <stdbool.h>
+#include "kernel-lib/list_sort.h"
 #include "kernel-shared/zoned.h"
-#include "kernel-shared/volumes.h"
 #include "common/string-table.h"
 #include "common/utils.h"
 #include "common/help.h"
@@ -62,7 +62,6 @@ static int cmd_device_add(const struct cmd_struct *cmd,
 {
 	char	*mntpnt;
 	int i, fdmnt, ret = 0;
-	DIR	*dirstream = NULL;
 	bool discard = true;
 	bool force = false;
 	int last_dev;
@@ -75,7 +74,7 @@ static int cmd_device_add(const struct cmd_struct *cmd,
 		int c;
 		enum { GETOPT_VAL_ENQUEUE = GETOPT_VAL_FIRST };
 		static const struct option long_options[] = {
-			{ "nodiscard", optional_argument, NULL, 'K'},
+			{ "nodiscard", no_argument, NULL, 'K' },
 			{ "force", no_argument, NULL, 'f'},
 			{ "enqueue", no_argument, NULL, GETOPT_VAL_ENQUEUE},
 			{ NULL, 0, NULL, 0}
@@ -105,7 +104,7 @@ static int cmd_device_add(const struct cmd_struct *cmd,
 	last_dev = argc - 1;
 	mntpnt = argv[last_dev];
 
-	fdmnt = btrfs_open_dir(mntpnt, &dirstream, 1);
+	fdmnt = btrfs_open_dir(mntpnt);
 	if (fdmnt < 0)
 		return 1;
 
@@ -113,7 +112,7 @@ static int cmd_device_add(const struct cmd_struct *cmd,
 	if (ret != 0) {
 		if (ret < 0)
 			error("unable to check status of exclusive operation: %m");
-		close_file_or_dir(fdmnt, dirstream);
+		close(fdmnt);
 		return 1;
 	}
 
@@ -170,7 +169,7 @@ static int cmd_device_add(const struct cmd_struct *cmd,
 		}
 
 		memset(&ioctl_args, 0, sizeof(ioctl_args));
-		strncpy_null(ioctl_args.name, path);
+		strncpy_null(ioctl_args.name, path, sizeof(ioctl_args.name));
 		res = ioctl(fdmnt, BTRFS_IOC_ADD_DEV, &ioctl_args);
 		if (res < 0) {
 			error("error adding device '%s': %m", path);
@@ -181,7 +180,7 @@ static int cmd_device_add(const struct cmd_struct *cmd,
 
 error_out:
 	btrfs_warn_multiple_profiles(fdmnt);
-	close_file_or_dir(fdmnt, dirstream);
+	close(fdmnt);
 	return !!ret;
 }
 static DEFINE_SIMPLE_COMMAND(device_add, "add");
@@ -191,7 +190,6 @@ static int _cmd_device_remove(const struct cmd_struct *cmd,
 {
 	char	*mntpnt;
 	int i, fdmnt, ret = 0;
-	DIR	*dirstream = NULL;
 	bool enqueue = false;
 	bool cancel = false;
 	bool force = false;
@@ -227,16 +225,15 @@ static int _cmd_device_remove(const struct cmd_struct *cmd,
 
 	mntpnt = argv[argc - 1];
 
-	fdmnt = btrfs_open_dir(mntpnt, &dirstream, 1);
+	fdmnt = btrfs_open_dir(mntpnt);
 	if (fdmnt < 0)
 		return 1;
 
 	/* Scan device arguments for 'cancel', that must be the only "device" */
 	for (i = optind; i < argc - 1; i++) {
 		if (cancel) {
-			error("cancel requested but another device specified: %s\n",
-				argv[i]);
-			close_file_or_dir(fdmnt, dirstream);
+			error("cancel requested but another device specified: %s", argv[i]);
+			close(fdmnt);
 			return 1;
 		}
 		if (strcmp("cancel", argv[i]) == 0) {
@@ -271,7 +268,7 @@ static int _cmd_device_remove(const struct cmd_struct *cmd,
 			if (ret < 0)
 				error(
 			"unable to check status of exclusive operation: %m");
-			close_file_or_dir(fdmnt, dirstream);
+			close(fdmnt);
 			return 1;
 		}
 	}
@@ -289,7 +286,7 @@ static int _cmd_device_remove(const struct cmd_struct *cmd,
 		} else if (strcmp(argv[i], "missing") == 0 ||
 			   cancel ||
 			   path_is_block_device(argv[i]) == 1) {
-			strncpy_null(argv2.name, argv[i]);
+			strncpy_null(argv2.name, argv[i], sizeof(argv2.name));
 		} else {
 			error("not a block device: %s", argv[i]);
 			ret++;
@@ -314,7 +311,7 @@ static int _cmd_device_remove(const struct cmd_struct *cmd,
 				continue;
 			}
 			memset(&arg, 0, sizeof(arg));
-			strncpy_null(arg.name, argv[i]);
+			strncpy_null(arg.name, argv[i], sizeof(arg.name));
 			res = ioctl(fdmnt, BTRFS_IOC_RM_DEV, &arg);
 		}
 
@@ -337,7 +334,7 @@ static int _cmd_device_remove(const struct cmd_struct *cmd,
 	}
 
 	btrfs_warn_multiple_profiles(fdmnt);
-	close_file_or_dir(fdmnt, dirstream);
+	close(fdmnt);
 	return !!ret;
 }
 
@@ -398,7 +395,7 @@ static int btrfs_forget_devices(const char *path)
 
 	memset(&args, 0, sizeof(args));
 	if (path)
-		strncpy_null(args.name, path);
+		strncpy_null(args.name, path, sizeof(args.name));
 	ret = ioctl(fd, BTRFS_IOC_FORGET_DEV, &args);
 	if (ret)
 		ret = -errno;
@@ -559,7 +556,7 @@ static int cmd_device_ready(const struct cmd_struct *cmd, int argc, char **argv)
 	}
 
 	memset(&args, 0, sizeof(args));
-	strncpy_null(args.name, path);
+	strncpy_null(args.name, path, sizeof(args.name));
 	ret = ioctl(fd, BTRFS_IOC_DEVICES_READY, &args);
 	if (ret < 0) {
 		error("unable to determine if device '%s' is ready for mount: %m",
@@ -589,10 +586,13 @@ static const char * const cmd_device_stats_usage[] = {
 	"btrfs device stats [options] <path>|<device>",
 	"Show device IO error statistics",
 	"Show device IO error statistics for all devices of the given filesystem",
-	"identified by PATH or DEVICE. The filesystem must be mounted.",
+	"identified by PATH or DEVICE. On a mounted filesystem the stats are read",
+	"using ioctls, for direct read from file images or block devices use the",
+	"option --offline.",
 	"",
 	OPTLINE("-c|--check", "return non-zero if any stat counter is not zero"),
 	OPTLINE("-z|--reset", "show current stats and reset values to zero"),
+	OPTLINE("--offline", "read stats from file or device directly (not from mounted filesystem)"),
 	OPTLINE("-T", "show current stats in tabular format"),
 	HELPINFO_INSERT_GLOBALS,
 	HELPINFO_INSERT_FORMAT,
@@ -627,7 +627,7 @@ static int print_device_stat_string(struct format_ctx *fctx,
 		canonical_path = malloc(32);
 
 		if (!canonical_path) {
-			error_msg(ERROR_MSG_MEMORY, "device path buffer");
+			error_mem("device path buffer");
 			return -ENOMEM;
 		}
 
@@ -681,7 +681,7 @@ static int print_device_stat_tabular(struct string_table *table, int row,
 		canonical_path = malloc(32);
 
 		if (!canonical_path) {
-			error_msg(ERROR_MSG_MEMORY, "device path buffer");
+			error_mem("device path buffer");
 			return -ENOMEM;
 		}
 
@@ -705,29 +705,132 @@ static int print_device_stat_tabular(struct string_table *table, int row,
 	return err;
 }
 
+/*
+ * Offline version of GET_DEV_STATS ioctl.
+ */
+static int get_device_stats_offline(struct btrfs_fs_info *fs_info, u64 devid,
+				    struct btrfs_ioctl_get_dev_stats *stats_args)
+{
+	int ret = 0;
+	struct btrfs_path path = { 0 };
+	struct btrfs_key key;
+	struct btrfs_dev_stats_item *stats_item;
+
+	key.objectid = BTRFS_DEV_STATS_OBJECTID;
+	key.type = BTRFS_PERSISTENT_ITEM_KEY;
+	key.offset = devid;
+	ret = btrfs_search_slot(NULL, fs_info->dev_root, &key, &path, 0, 0);
+	if (ret < 0)
+		goto out;
+	if (ret > 0) {
+		pr_verbose(LOG_DEBUG, "no device stats found for devid %llu\n", devid);
+		ret = 0;
+		goto out;
+	}
+
+	memset(stats_args->values, 0, stats_args->nr_items * sizeof(u64));
+	stats_item = btrfs_item_ptr(path.nodes[0], path.slots[0], struct btrfs_dev_stats_item);
+
+	stats_args->values[BTRFS_DEV_STAT_WRITE_ERRS] =
+		btrfs_dev_stats_value(path.nodes[0], stats_item, BTRFS_DEV_STAT_WRITE_ERRS);
+	stats_args->values[BTRFS_DEV_STAT_READ_ERRS] =
+		btrfs_dev_stats_value(path.nodes[0], stats_item, BTRFS_DEV_STAT_READ_ERRS);
+	stats_args->values[BTRFS_DEV_STAT_FLUSH_ERRS] =
+		btrfs_dev_stats_value(path.nodes[0], stats_item, BTRFS_DEV_STAT_FLUSH_ERRS);
+	stats_args->values[BTRFS_DEV_STAT_CORRUPTION_ERRS] =
+		btrfs_dev_stats_value(path.nodes[0], stats_item, BTRFS_DEV_STAT_CORRUPTION_ERRS);
+	stats_args->values[BTRFS_DEV_STAT_GENERATION_ERRS] =
+		btrfs_dev_stats_value(path.nodes[0], stats_item, BTRFS_DEV_STAT_GENERATION_ERRS);
+
+out:
+	btrfs_release_path(&path);
+	return ret;
+}
+
+/*
+ * Offline version of get_fs_info() with limitations:
+ *
+ * - no seeding device support
+ * - fi_args filled only partially (num_devices)
+ */
+static int get_fs_info_offline(struct btrfs_fs_info *fs_info,
+			       struct btrfs_ioctl_fs_info_args *fi_args,
+			       struct btrfs_ioctl_dev_info_args **di_ret)
+{
+	int i;
+	struct btrfs_ioctl_dev_info_args *di_args;
+	struct btrfs_fs_devices *cur_fs;
+	struct btrfs_device *device;
+	struct list_head *fs_uuids;
+
+	fs_uuids = btrfs_scanned_uuids();
+	/* Count devices from fs_devices in case some of them are missing */
+	fi_args->num_devices = 0;
+	list_for_each_entry(cur_fs, fs_uuids, fs_list) {
+		if (memcmp(fs_info->fs_devices->fsid, cur_fs->fsid, BTRFS_FSID_SIZE) != 0)
+			continue;
+		list_for_each_entry(device, &cur_fs->devices, dev_list) {
+			fi_args->num_devices++;
+		}
+	}
+
+	di_args = malloc(fi_args->num_devices * sizeof(*di_args));
+	if (!di_args)
+		return -ENOMEM;
+
+	i = 0;
+	list_sort(NULL, &fs_info->fs_devices->devices, cmp_device_id);
+	list_for_each_entry(cur_fs, fs_uuids, fs_list) {
+		if (memcmp(fs_info->fs_devices->fsid, cur_fs->fsid, BTRFS_FSID_SIZE) != 0)
+			continue;
+
+		list_for_each_entry(device, &cur_fs->devices, dev_list) {
+			di_args[i].devid = device->devid;
+			memcpy(di_args[i].uuid, device->uuid, BTRFS_FSID_SIZE);
+			di_args[i].bytes_used = device->bytes_used;
+			di_args[i].total_bytes = device->total_bytes;
+			memcpy(di_args[i].fsid, cur_fs->fsid, BTRFS_FSID_SIZE);
+			/*
+			 * File devices without loop device or a missing device
+			 * don't have a path.
+			 */
+			if (device->name)
+				strncpy_null((char *)di_args[i].path, device->name,
+					     BTRFS_DEVICE_PATH_NAME_MAX + 1);
+			i++;
+		}
+	}
+
+	*di_ret = di_args;
+	return 0;
+}
+
 static int cmd_device_stats(const struct cmd_struct *cmd, int argc, char **argv)
 {
 	char *dev_path;
 	struct btrfs_ioctl_fs_info_args fi_args;
 	struct btrfs_ioctl_dev_info_args *di_args = NULL;
+	struct btrfs_fs_info *fs_info = NULL;
 	struct string_table *table = NULL;
 	int ret;
-	int fdmnt;
+	int fdmnt = -1;
 	int i;
 	int err = 0;
 	bool check = false;
 	bool free_table = false;
 	bool tabular = false;
+	bool opt_offline = false;
 	__u64 flags = 0;
-	DIR *dirstream = NULL;
 	struct format_ctx fctx;
 
 	optind = 0;
 	while (1) {
 		int c;
+		enum { GETOPT_VAL_OFFLINE = GETOPT_VAL_FIRST };
 		static const struct option long_options[] = {
 			{"check", no_argument, NULL, 'c'},
 			{"reset", no_argument, NULL, 'z'},
+			{"offline", no_argument, NULL, GETOPT_VAL_OFFLINE},
 			{NULL, 0, NULL, 0}
 		};
 
@@ -745,6 +848,9 @@ static int cmd_device_stats(const struct cmd_struct *cmd, int argc, char **argv)
 		case 'T':
 			tabular = true;
 			break;
+		case GETOPT_VAL_OFFLINE:
+			opt_offline = true;
+			break;
 		default:
 			usage_unknown_option(cmd, argv);
 		}
@@ -753,19 +859,48 @@ static int cmd_device_stats(const struct cmd_struct *cmd, int argc, char **argv)
 	if (check_argc_exact(argc - optind, 1))
 		return 1;
 
+	if (opt_offline && (flags & BTRFS_DEV_STATS_RESET)) {
+		error("--offline and --reset cannot be used together");
+		return 1;
+	}
+
 	dev_path = argv[optind];
 
-	fdmnt = open_path_or_dev_mnt(dev_path, &dirstream, 1);
-	if (fdmnt < 0)
-		return 1;
+	if (!opt_offline) {
+		fdmnt = btrfs_open_mnt(dev_path);
+		if (fdmnt < 0) {
+			if (fdmnt == -ENOTDIR && !opt_offline)
+				error("to read device stats from a file image use --offline option");
+			return 1;
+		}
 
-	ret = get_fs_info(dev_path, &fi_args, &di_args);
-	if (ret) {
-		errno = -ret;
-		error("getting device info for %s failed: %m", dev_path);
-		err = 1;
-		goto out;
+		ret = get_fs_info(dev_path, &fi_args, &di_args);
+		if (ret) {
+			errno = -ret;
+			error("getting device info for %s failed: %m", dev_path);
+			err = 1;
+			goto out;
+		}
+	} else {
+		struct open_ctree_args oca = { 0 };
+
+		oca.filename = dev_path;
+		fs_info = open_ctree_fs_info(&oca);
+		if (!fs_info) {
+			error("cannot open filesystem on %s", dev_path);
+			err = 1;
+			goto out;
+		}
+
+		ret = get_fs_info_offline(fs_info, &fi_args, &di_args);
+		if (ret < 0) {
+			errno = -ret;
+			error("getting device info for %s failed: %m", dev_path);
+			err = 1;
+			goto out;
+		}
 	}
+
 	if (!fi_args.num_devices) {
 		error("no devices found");
 		err = 1;
@@ -779,7 +914,7 @@ static int cmd_device_stats(const struct cmd_struct *cmd, int argc, char **argv)
 		 */
 		table = table_create(7, fi_args.num_devices + 2);
 		if (!table) {
-			error_msg(ERROR_MSG_MEMORY, NULL);
+			error_mem(NULL);
 			goto out;
 		}
 		free_table = true;
@@ -802,19 +937,33 @@ static int cmd_device_stats(const struct cmd_struct *cmd, int argc, char **argv)
 		char path[BTRFS_DEVICE_PATH_NAME_MAX + 1];
 		int err2;
 
-		strncpy(path, (char *)di_args[i].path,
-			BTRFS_DEVICE_PATH_NAME_MAX);
-		path[BTRFS_DEVICE_PATH_NAME_MAX] = 0;
+		strncpy_null(path, (char *)di_args[i].path,
+			     BTRFS_DEVICE_PATH_NAME_MAX + 1);
 
 		args.devid = di_args[i].devid;
 		args.nr_items = BTRFS_DEV_STAT_VALUES_MAX;
-		args.flags = flags;
+		if (!opt_offline) {
+			args.flags = flags;
 
-		if (ioctl(fdmnt, BTRFS_IOC_GET_DEV_STATS, &args) < 0) {
-			error("device stats ioctl failed on %s: %m",
-			      path);
-			err |= 1;
-			goto out;
+			if (ioctl(fdmnt, BTRFS_IOC_GET_DEV_STATS, &args) < 0) {
+				error("device stats ioctl failed on %s: %m",
+				      path);
+				err |= 1;
+				goto out;
+			}
+		} else {
+			if (args.flags != 0) {
+				error("no flags supported");
+				err = -EINVAL;
+				goto out;
+			}
+			ret = get_device_stats_offline(fs_info, di_args[i].devid, &args);
+			if (ret < 0) {
+				error("cannot read offline stats for devid %llu",
+				      di_args[i].devid);
+				err = ret;
+				goto out;
+			}
 		}
 
 		if (tabular)
@@ -841,7 +990,9 @@ static int cmd_device_stats(const struct cmd_struct *cmd, int argc, char **argv)
 
 out:
 	free(di_args);
-	close_file_or_dir(fdmnt, dirstream);
+	close(fdmnt);
+	if (fs_info)
+		close_ctree_fs_info(fs_info);
 	if (free_table)
 		table_free(table);
 
@@ -900,12 +1051,11 @@ static int cmd_device_usage(const struct cmd_struct *cmd, int argc, char **argv)
 
 	for (i = optind; i < argc; i++) {
 		int fd;
-		DIR *dirstream = NULL;
 
 		if (i > 1)
 			pr_verbose(LOG_DEFAULT, "\n");
 
-		fd = btrfs_open_dir(argv[i], &dirstream, 1);
+		fd = btrfs_open_dir(argv[i]);
 		if (fd < 0) {
 			ret = 1;
 			break;
@@ -913,7 +1063,7 @@ static int cmd_device_usage(const struct cmd_struct *cmd, int argc, char **argv)
 
 		ret = _cmd_device_usage(fd, argv[i], unit_mode);
 		btrfs_warn_multiple_profiles(fd);
-		close_file_or_dir(fd, dirstream);
+		close(fd);
 
 		if (ret)
 			break;
