@@ -456,8 +456,8 @@ static void print_table_head(void)
 		max_len = btrfs_qgroup_columns[i].max_len;
 		if (!btrfs_qgroup_columns[i].need_print)
 			continue;
-		if ((i == BTRFS_QGROUP_QGROUPID) | (i == BTRFS_QGROUP_PARENT) |
-			(i == BTRFS_QGROUP_CHILD))
+		if ((i == BTRFS_QGROUP_QGROUPID) || (i == BTRFS_QGROUP_PARENT) ||
+		    (i == BTRFS_QGROUP_CHILD))
 			printf("%-*s", max_len, btrfs_qgroup_columns[i].column_name);
 		else
 			printf("%*s", max_len, btrfs_qgroup_columns[i].column_name);
@@ -468,8 +468,8 @@ static void print_table_head(void)
 		max_len = btrfs_qgroup_columns[i].max_len;
 		if (!btrfs_qgroup_columns[i].need_print)
 			continue;
-		if ((i == BTRFS_QGROUP_QGROUPID) | (i == BTRFS_QGROUP_PARENT) |
-			(i == BTRFS_QGROUP_CHILD)) {
+		if ((i == BTRFS_QGROUP_QGROUPID) || (i == BTRFS_QGROUP_PARENT) ||
+		    (i == BTRFS_QGROUP_CHILD)) {
 			len = strlen(btrfs_qgroup_columns[i].column_name);
 			while (len--)
 				printf("-");
@@ -662,7 +662,7 @@ static struct btrfs_qgroup_comparer_set *qgroup_alloc_comparer_set(void)
 	       sizeof(struct btrfs_qgroup_comparer);
 	set = calloc(1, size);
 	if (!set) {
-		error_msg(ERROR_MSG_MEMORY, NULL);
+		error_mem(NULL);
 		exit(1);
 	}
 
@@ -822,7 +822,7 @@ static struct btrfs_qgroup *get_or_add_qgroup(int fd,
 
 	bq = calloc(1, sizeof(*bq));
 	if (!bq) {
-		error_msg(ERROR_MSG_MEMORY, NULL);
+		error_mem(NULL);
 		return ERR_PTR(-ENOMEM);
 	}
 
@@ -939,7 +939,7 @@ static int update_qgroup_relation(struct qgroup_lookup *qgroup_lookup,
 
 	list = malloc(sizeof(*list));
 	if (!list) {
-		error_msg(ERROR_MSG_MEMORY, NULL);
+		error_mem(NULL);
 		return -ENOMEM;
 	}
 
@@ -970,8 +970,7 @@ static void __free_btrfs_qgroup(struct btrfs_qgroup *bq)
 		list_del(&list->next_member);
 		free(list);
 	}
-	if (bq->path)
-		free((void *)bq->path);
+	free((void *)bq->path);
 	free(bq);
 }
 
@@ -1075,7 +1074,7 @@ static struct btrfs_qgroup_filter_set *qgroup_alloc_filter_set(void)
 	       sizeof(struct btrfs_qgroup_filter);
 	set = calloc(1, size);
 	if (!set) {
-		error_msg(ERROR_MSG_MEMORY, NULL);
+		error_mem(NULL);
 		exit(1);
 	}
 	set->total = BTRFS_QGROUP_NFILTERS_INCREASE;
@@ -1102,7 +1101,7 @@ static int qgroup_setup_filter(struct btrfs_qgroup_filter_set **filter_set,
 		tmp = set;
 		set = realloc(set, size);
 		if (!set) {
-			error_msg(ERROR_MSG_MEMORY, NULL);
+			error_mem(NULL);
 			free(tmp);
 			exit(1);
 		}
