@@ -24,7 +24,6 @@
 #include <string.h>
 #include <unistd.h>
 #include <errno.h>
-#include <libgen.h>
 #include <getopt.h>
 #include <dirent.h>
 #include <stdbool.h>
@@ -46,6 +45,8 @@
 #include "common/string-utils.h"
 #include "common/units.h"
 #include "common/format-output.h"
+#include "common/tree-search.h"
+#include "common/parse-utils.h"
 #include "cmds/commands.h"
 #include "cmds/qgroup.h"
 
@@ -91,7 +92,7 @@ static int wait_for_subvolume_cleaning(int fd, size_t count, uint64_t *ids,
 		for (i = 0; i < count; i++) {
 			if (!ids[i])
 				continue;
-			err = btrfs_util_subvolume_info_fd(fd, ids[i], NULL);
+			err = btrfs_util_subvolume_get_info_fd(fd, ids[i], NULL);
 			if (err == BTRFS_UTIL_ERROR_SUBVOLUME_NOT_FOUND) {
 				done++;
 				pr_verbose(LOG_DEFAULT, "Subvolume id %" PRIu64 " is gone (%zu/%zu)\n",
@@ -140,60 +141,27 @@ static const char * const cmd_subvolume_create_usage[] = {
 	NULL
 };
 
-static int create_one_subvolume(const char *dst, struct btrfs_qgroup_inherit *inherit,
+static int create_one_subvolume(const char *dst, struct btrfs_util_qgroup_inherit *inherit,
 				bool create_parents)
 {
 	int ret;
-	int len;
-	int	fddst = -1;
-	char	*dupname = NULL;
-	char	*dupdir = NULL;
-	char	*newname;
-	char	*dstdir;
-	DIR	*dirstream = NULL;
+	char dupname[PATH_MAX];
+	char dupdir[PATH_MAX];
+	const char *newname;
+	char *dstdir;
+	enum btrfs_util_error err;
 
-	ret = path_is_dir(dst);
-	if (ret < 0 && ret != -ENOENT) {
-		errno = -ret;
-		error("cannot access %s: %m", dst);
-		goto out;
-	}
-	if (ret >= 0) {
-		error("target path already exists: %s", dst);
-		goto out;
-	}
-
-	dupname = strdup(dst);
-	if (!dupname) {
-		error_msg(ERROR_MSG_MEMORY, "duplicating %s", dst);
-		goto out;
-	}
-	newname = basename(dupname);
-
-	dupdir = strdup(dst);
-	if (!dupdir) {
-		error_msg(ERROR_MSG_MEMORY, "duplicating %s", dst);
-		goto out;
-	}
-	dstdir = dirname(dupdir);
-
-	if (!test_issubvolname(newname)) {
-		error("invalid subvolume name: %s", newname);
-		goto out;
-	}
-
-	len = strlen(newname);
-	if (len > BTRFS_VOL_NAME_MAX) {
-		error("subvolume name too long: %s", newname);
-		goto out;
-	}
+	strncpy_null(dupname, dst, PATH_MAX);
+	newname = path_basename(dupname);
+	strncpy_null(dupdir, dst, PATH_MAX);
+	dstdir = path_dirname(dupdir);
 
 	if (create_parents) {
 		char p[PATH_MAX] = { 0 };
 		char dstdir_dup[PATH_MAX];
 		char *token;
 
-		strncpy_null(dstdir_dup, dstdir);
+		strncpy_null(dstdir_dup, dstdir, sizeof(dstdir_dup));
 		if (dstdir_dup[0] == '/')
 			strcat(p, "/");
 
@@ -208,6 +176,8 @@ static int create_one_subvolume(const char *dst, struct btrfs_qgroup_inherit *in
 					goto out;
 				}
 			} else if (ret <= 0) {
+				if (ret == 0)
+					ret = -EEXIST;
 				errno = ret ;
 				error("failed to check directory %s before creation: %m", p);
 				goto out;
@@ -217,46 +187,55 @@ static int create_one_subvolume(const char *dst, struct btrfs_qgroup_inherit *in
 		}
 	}
 
-	fddst = btrfs_open_dir(dstdir, &dirstream, 1);
-	if (fddst < 0)
+	err = btrfs_util_subvolume_create(dst, 0, NULL, inherit);
+	if (err) {
+		error_btrfs_util(err);
+		ret = -errno;
 		goto out;
+	}
 
 	pr_verbose(LOG_DEFAULT, "Create subvolume '%s/%s'\n", dstdir, newname);
-	if (inherit) {
-		struct btrfs_ioctl_vol_args_v2	args;
 
-		memset(&args, 0, sizeof(args));
-		strncpy_null(args.name, newname);
-		args.flags |= BTRFS_SUBVOL_QGROUP_INHERIT;
-		args.size = btrfs_qgroup_inherit_size(inherit);
-		args.qgroup_inherit = inherit;
-
-		ret = ioctl(fddst, BTRFS_IOC_SUBVOL_CREATE_V2, &args);
-	} else {
-		struct btrfs_ioctl_vol_args	args;
-
-		memset(&args, 0, sizeof(args));
-		strncpy_null(args.name, newname);
-
-		ret = ioctl(fddst, BTRFS_IOC_SUBVOL_CREATE, &args);
-	}
-
-	if (ret < 0) {
-		error("cannot create subvolume: %m");
-		goto out;
-	}
+	ret = 0;
 
 out:
-	close_file_or_dir(fddst, dirstream);
-	free(dupname);
-	free(dupdir);
 
 	return ret;
 }
+
+static int qgroup_inherit_add_group(struct btrfs_util_qgroup_inherit **inherit,
+				    const char *arg)
+{
+	enum btrfs_util_error err;
+	u64 qgroupid;
+
+	if (!*inherit) {
+		err = btrfs_util_qgroup_inherit_create(0, inherit);
+		if (err) {
+			error_btrfs_util(err);
+			return -errno;
+		}
+	}
+
+	qgroupid = parse_qgroupid_or_path(optarg);
+	if (qgroupid == 0) {
+		error("invalid qgroup specification, qgroupid must not be 0");
+		return -EINVAL;
+	}
+
+	err = btrfs_util_qgroup_inherit_add_group(inherit, qgroupid);
+	if (err) {
+		error_btrfs_util(err);
+		return -errno;
+	}
+
+	return 0;
+}
+
 static int cmd_subvolume_create(const struct cmd_struct *cmd, int argc, char **argv)
 {
 	int retval, ret;
-	struct btrfs_qgroup_inherit *inherit = NULL;
+	struct btrfs_util_qgroup_inherit *inherit = NULL;
 	bool has_error = false;
 	bool create_parents = false;
 
@@ -273,15 +252,8 @@ static int cmd_subvolume_create(const struct cmd_struct *cmd, int argc, char **a
 			break;
 
 		switch (c) {
-		case 'c':
-			ret = btrfs_qgroup_inherit_add_copy(&inherit, optarg, 0);
-			if (ret) {
-				retval = ret;
-				goto out;
-			}
-			break;
 		case 'i':
-			ret = btrfs_qgroup_inherit_add_group(&inherit, optarg);
+			ret = qgroup_inherit_add_group(&inherit, optarg);
 			if (ret) {
 				retval = ret;
 				goto out;
@@ -310,7 +282,7 @@ static int cmd_subvolume_create(const struct cmd_struct *cmd, int argc, char **a
 	if (!has_error)
 		retval = 0;
 out:
-	free(inherit);
+	btrfs_util_qgroup_inherit_destroy(inherit);
 
 	return retval;
 }
@@ -321,11 +293,11 @@ static int wait_for_commit(int fd)
 	enum btrfs_util_error err;
 	uint64_t transid;
 
-	err = btrfs_util_start_sync_fd(fd, &transid);
+	err = btrfs_util_fs_start_sync_fd(fd, &transid);
 	if (err)
 		return -1;
 
-	err = btrfs_util_wait_sync_fd(fd, transid);
+	err = btrfs_util_fs_wait_sync_fd(fd, transid);
 	if (err)
 		return -1;
 
@@ -347,8 +319,8 @@ static const char * const cmd_subvolume_delete_usage[] = {
 	OPTLINE("-c|--commit-after", "wait for transaction commit at the end of the operation"),
 	OPTLINE("-C|--commit-each", "wait for transaction commit after deleting each subvolume"),
 	OPTLINE("-i|--subvolid", "subvolume id of the to be removed subvolume"),
-	OPTLINE("--delete-qgroup", "also delete the qgroup 0/subvolid if it exists"),
-	OPTLINE("--no-delete-qgroup", "do not delete the qgroup 0/subvolid if it exists (default)"),
+	OPTLINE("-R|--recursive", "delete accessible subvolumes beneath each subvolume recursively, "
+		"this is not atomic, may need root to delete subvolumes not accessible by the user"),
 	OPTLINE("-v|--verbose", "deprecated, alias for global -v option"),
 	HELPINFO_INSERT_GLOBALS,
 	HELPINFO_INSERT_VERBOSE,
@@ -362,13 +334,14 @@ static int cmd_subvolume_delete(const struct cmd_struct *cmd, int argc, char **a
 	int res, ret = 0;
 	int cnt;
 	int fd = -1;
-	char	*dname, *vname, *cpath;
+	char	*dname, *cpath;
+	const char *vname;
 	char	*dupdname = NULL;
 	char	*dupvname = NULL;
 	char	*path = NULL;
-	DIR	*dirstream = NULL;
 	int commit_mode = 0;
 	bool subvol_path_not_found = false;
+	int flags = 0;
 	u8 fsid[BTRFS_FSID_SIZE];
 	u64 subvolid = 0;
 	char uuidbuf[BTRFS_UUID_UNPARSED_SIZE];
@@ -377,24 +350,20 @@ static int cmd_subvolume_delete(const struct cmd_struct *cmd, int argc, char **a
 	enum { COMMIT_AFTER = 1, COMMIT_EACH = 2 };
 	enum btrfs_util_error err;
 	uint64_t default_subvol_id, target_subvol_id = 0;
-	bool opt_delete_qgroup = false;
 
 	optind = 0;
 	while (1) {
 		int c;
-		enum { GETOPT_VAL_DELETE_QGROUP = GETOPT_VAL_FIRST,
-		       GETOPT_VAL_NO_DELETE_QGROUP };
 		static const struct option long_options[] = {
 			{"commit-after", no_argument, NULL, 'c'},
 			{"commit-each", no_argument, NULL, 'C'},
 			{"subvolid", required_argument, NULL, 'i'},
+			{"recursive", no_argument, NULL, 'R'},
 			{"verbose", no_argument, NULL, 'v'},
-			{"delete-qgroup", no_argument, NULL, GETOPT_VAL_DELETE_QGROUP },
-			{"no-delete-qgroup", no_argument, NULL, GETOPT_VAL_NO_DELETE_QGROUP },
 			{NULL, 0, NULL, 0}
 		};
 
-		c = getopt_long(argc, argv, "cCi:v", long_options, NULL);
+		c = getopt_long(argc, argv, "cCi:Rv", long_options, NULL);
 		if (c < 0)
 			break;
 
@@ -408,15 +377,12 @@ static int cmd_subvolume_delete(const struct cmd_struct *cmd, int argc, char **a
 		case 'i':
 			subvolid = arg_strtou64(optarg);
 			break;
+		case 'R':
+			flags |= BTRFS_UTIL_DELETE_SUBVOLUME_RECURSIVE;
+			break;
 		case 'v':
 			bconf_be_verbose();
 			break;
-		case GETOPT_VAL_DELETE_QGROUP:
-			opt_delete_qgroup = true;
-			break;
-		case GETOPT_VAL_NO_DELETE_QGROUP:
-			opt_delete_qgroup = false;
-			break;
 		default:
 			usage_unknown_option(cmd, argv);
 		}
@@ -429,6 +395,11 @@ static int cmd_subvolume_delete(const struct cmd_struct *cmd, int argc, char **a
 	if (subvolid > 0 && check_argc_exact(argc - optind, 1))
 		return 1;
 
+	if (subvolid > 0 && flags & BTRFS_UTIL_DELETE_SUBVOLUME_RECURSIVE) {
+		error("option --recursive is not supported with --subvolid");
+		return 1;
+	}
+
 	pr_verbose(LOG_INFO, "Transaction commit: %s\n",
 		   !commit_mode ? "none (default)" :
 		   commit_mode == COMMIT_AFTER ? "at the end" : "after each");
@@ -440,7 +411,7 @@ static int cmd_subvolume_delete(const struct cmd_struct *cmd, int argc, char **a
 		char *subvol;
 
 		path = argv[cnt];
-		err = btrfs_util_subvolume_path(path, subvolid, &subvol);
+		err = btrfs_util_subvolume_get_path(path, subvolid, &subvol);
 		/*
 		 * If the subvolume is really not referred by anyone, and refs
 		 * is 0, newer kernel can handle it by just adding an orphan
@@ -467,7 +438,7 @@ static int cmd_subvolume_delete(const struct cmd_struct *cmd, int argc, char **a
 again:
 	path = argv[cnt];
 
-	err = btrfs_util_is_subvolume(path);
+	err = btrfs_util_subvolume_is_valid(path);
 	if (err) {
 		error_btrfs_util(err);
 		ret = 1;
@@ -481,32 +452,32 @@ again:
 		goto out;
 	}
 	dupdname = strdup(cpath);
-	dname = dirname(dupdname);
+	dname = path_dirname(dupdname);
 	dupvname = strdup(cpath);
-	vname = basename(dupvname);
+	vname = path_basename(dupvname);
 	free(cpath);
 
 	/* When subvolid is passed, <path> will point to the mount point */
 	if (subvolid > 0)
 		dname = dupvname;
 
-	fd = btrfs_open_dir(dname, &dirstream, 1);
+	fd = btrfs_open_dir(dname);
 	if (fd < 0) {
 		ret = 1;
 		goto out;
 	}
 
 	default_subvol_id = 0;
-	err = btrfs_util_get_default_subvolume_fd(fd, &default_subvol_id);
+	err = btrfs_util_subvolume_get_default_fd(fd, &default_subvol_id);
 	if (err == BTRFS_UTIL_ERROR_SEARCH_FAILED) {
-		if (geteuid() != 0)
+		if (geteuid() == 0)
 			warning("cannot read default subvolume id: %m");
 	}
 
 	if (subvolid > 0) {
 		target_subvol_id = subvolid;
 	} else {
-		err = btrfs_util_subvolume_id(path, &target_subvol_id);
+		err = btrfs_util_subvolume_get_id(path, &target_subvol_id);
 		if (err) {
 			ret = 1;
 			goto out;
@@ -523,6 +494,35 @@ again:
 		goto out;
 	}
 
+	if (flags & BTRFS_UTIL_DELETE_SUBVOLUME_RECURSIVE) {
+		struct btrfs_util_subvolume_iterator *iter;
+
+		err = btrfs_util_subvolume_iter_create_fd(fd, target_subvol_id,
+				  BTRFS_UTIL_SUBVOLUME_ITERATOR_POST_ORDER,
+				  &iter);
+		if (!err) {
+			char *nested_path;
+			struct btrfs_util_subvolume_info subvol_info;
+
+			while (!(err = btrfs_util_subvolume_iter_next_info(iter, &nested_path, &subvol_info))) {
+				pr_verbose(LOG_DEFAULT, "Delete subvolume %" PRIu64 " (%s): ",
+					   subvol_info.id,
+					   commit_mode == COMMIT_EACH ||
+					   (commit_mode == COMMIT_AFTER && cnt + 1 == argc) ?
+					   "commit" : "no-commit");
+				pr_verbose(LOG_DEFAULT, "'%s/%s/%s'\n", dname, vname, nested_path);
+
+				free(nested_path);
+			}
+			if (err != BTRFS_UTIL_ERROR_STOP_ITERATION)
+				warning("failed to iterate subvolumes, nested subvolumes will not be printed: %s", btrfs_util_strerror(err));
+
+			btrfs_util_subvolume_iter_destroy(iter);
+		} else {
+			warning("failed to create subvolume iterator, nested subvolumes will not be printed: %s", btrfs_util_strerror(err));
+		}
+	}
+
 	pr_verbose(LOG_DEFAULT, "Delete subvolume %" PRIu64 " (%s): ",
 		target_subvol_id,
 		commit_mode == COMMIT_EACH ||
@@ -541,30 +541,17 @@ again:
 
 	/* Start deleting. */
 	if (subvolid == 0)
-		err = btrfs_util_delete_subvolume_fd(fd, vname, 0);
+		err = btrfs_util_subvolume_delete_fd(fd, vname, flags);
 	else
-		err = btrfs_util_delete_subvolume_by_id_fd(fd, subvolid);
+		err = btrfs_util_subvolume_delete_by_id_fd(fd, subvolid);
 	if (err) {
 		int saved_errno = errno;
 
 		error_btrfs_util(err);
 		if (saved_errno == EPERM)
-			warning("deletion failed with EPERM, you don't have permissions or send may be in progress");
+			warning("deletion failed with EPERM, you don't have permissions or send may be in progress or the subvolume is set as default");
 		ret = 1;
 		goto out;
-	} else if (opt_delete_qgroup) {
-		struct btrfs_ioctl_qgroup_create_args args = { .qgroupid = target_subvol_id };
-
-		ret = ioctl(fd, BTRFS_IOC_QGROUP_CREATE, &args);
-		if (ret == 0) {
-			pr_verbose(LOG_DEFAULT, "Delete qgroup 0/%" PRIu64 "\n", target_subvol_id);
-		} else if (ret < 0 && (errno == ENOTCONN || errno == ENOENT)) {
-			/* Quotas not enabled, or there's no qgroup. */
-		} else {
-			warning("unable to delete qgroup 0/%llu: %m", subvolid);
-		}
-		/* Qgroup errors are not fatal. */
-		ret = 0;
 	}
 
 	if (commit_mode == COMMIT_EACH) {
@@ -584,7 +571,7 @@ again:
 			goto out;
 		}
 
-		if (add_seen_fsid(fsid, seen_fsid_hash, fd, dirstream) == 0) {
+		if (add_seen_fsid(fsid, seen_fsid_hash, fd) == 0) {
 			uuid_unparse(fsid, uuidbuf);
 			pr_verbose(LOG_INFO, "  new fs is found for '%s', fsid: %s\n",
 				   path, uuidbuf);
@@ -598,10 +585,9 @@ again:
 	}
 
 out:
-	close_file_or_dir(fd, dirstream);
+	close(fd);
 keep_fd:
 	fd = -1;
-	dirstream = NULL;
 	free(dupdname);
 	free(dupvname);
 	dupdname = NULL;
@@ -665,49 +651,27 @@ static int cmd_subvolume_snapshot(const struct cmd_struct *cmd, int argc, char *
 {
 	char	*subvol, *dst;
 	int	res, retval;
-	int	fd = -1, fddst = -1;
-	int	len;
-	bool readonly = false;
-	char	*dupname = NULL;
-	char	*dupdir = NULL;
-	char	*newname;
-	char	*dstdir;
+	char	*dstdir = NULL;
 	enum btrfs_util_error err;
-	struct btrfs_ioctl_vol_args_v2	args;
-	struct btrfs_qgroup_inherit *inherit = NULL;
-	DIR *dirstream1 = NULL, *dirstream2 = NULL;
+	struct btrfs_util_qgroup_inherit *inherit = NULL;
+	int flags = 0;
 
-	memset(&args, 0, sizeof(args));
 	optind = 0;
 	while (1) {
-		int c = getopt(argc, argv, "c:i:r");
+		int c = getopt(argc, argv, "i:r");
 		if (c < 0)
 			break;
 
 		switch (c) {
-		case 'c':
-			res = btrfs_qgroup_inherit_add_copy(&inherit, optarg, 0);
-			if (res) {
-				retval = res;
-				goto out;
-			}
-			break;
 		case 'i':
-			res = btrfs_qgroup_inherit_add_group(&inherit, optarg);
+			res = qgroup_inherit_add_group(&inherit, optarg);
 			if (res) {
 				retval = res;
 				goto out;
 			}
 			break;
 		case 'r':
-			readonly = true;
-			break;
-		case 'x':
-			res = btrfs_qgroup_inherit_add_copy(&inherit, optarg, 1);
-			if (res) {
-				retval = res;
-				goto out;
-			}
+			flags |= BTRFS_UTIL_CREATE_SNAPSHOT_READ_ONLY;
 			break;
 		default:
 			usage_unknown_option(cmd, argv);
@@ -723,7 +687,7 @@ static int cmd_subvolume_snapshot(const struct cmd_struct *cmd, int argc, char *
 	dst = argv[optind + 1];
 
 	retval = 1;	/* failure */
-	err = btrfs_util_is_subvolume(subvol);
+	err = btrfs_util_subvolume_is_valid(subvol);
 	if (err) {
 		error_btrfs_util(err);
 		goto out;
@@ -741,71 +705,49 @@ static int cmd_subvolume_snapshot(const struct cmd_struct *cmd, int argc, char *
 	}
 
 	if (res > 0) {
+		char *dupname;
+		const char *newname;
+
 		dupname = strdup(subvol);
-		newname = basename(dupname);
-		dstdir = dst;
+		newname = path_basename(dupname);
+
+		dstdir = malloc(strlen(dst) + 1 + strlen(newname) + 1);
+		if (!dstdir) {
+			error_mem(NULL);
+			free(dupname);
+			goto out;
+		}
+
+		dstdir[0] = 0;
+		strcpy(dstdir, dst);
+		strcat(dstdir, "/");
+		strcat(dstdir, newname);
+
+		free(dupname);
 	} else {
-		dupname = strdup(dst);
-		newname = basename(dupname);
-		dupdir = strdup(dst);
-		dstdir = dirname(dupdir);
+		dstdir = strdup(dst);
 	}
 
-	if (!test_issubvolname(newname)) {
-		error("invalid snapshot name '%s'", newname);
-		goto out;
-	}
-
-	len = strlen(newname);
-	if (len > BTRFS_VOL_NAME_MAX) {
-		error("snapshot name too long '%s'", newname);
-		goto out;
-	}
-
-	fddst = btrfs_open_dir(dstdir, &dirstream1, 1);
-	if (fddst < 0)
-		goto out;
-
-	fd = btrfs_open_dir(subvol, &dirstream2, 1);
-	if (fd < 0)
-		goto out;
-
-	if (readonly) {
-		args.flags |= BTRFS_SUBVOL_RDONLY;
-		pr_verbose(LOG_DEFAULT,
-			   "Create a readonly snapshot of '%s' in '%s/%s'\n",
-			   subvol, dstdir, newname);
-	} else {
-		pr_verbose(LOG_DEFAULT,
-			   "Create a snapshot of '%s' in '%s/%s'\n",
-			   subvol, dstdir, newname);
-	}
-
-	args.fd = fd;
-	if (inherit) {
-		args.flags |= BTRFS_SUBVOL_QGROUP_INHERIT;
-		args.size = btrfs_qgroup_inherit_size(inherit);
-		args.qgroup_inherit = inherit;
-	}
-	strncpy_null(args.name, newname);
-
-	res = ioctl(fddst, BTRFS_IOC_SNAP_CREATE_V2, &args);
-	if (res < 0) {
-		if (errno == ETXTBSY)
-			error("cannot snapshot '%s': source subvolume contains an active swapfile (%m)", subvol);
-		else
-			error("cannot snapshot '%s': %m", subvol);
+	err = btrfs_util_subvolume_snapshot(subvol, dstdir, flags, NULL, inherit);
+	if (err) {
+		error_btrfs_util(err);
 		goto out;
 	}
 
 	retval = 0;	/* success */
 
+	if (flags & BTRFS_UTIL_CREATE_SNAPSHOT_READ_ONLY)
+		pr_verbose(LOG_DEFAULT,
+			   "Create readonly snapshot of '%s' in '%s'\n",
+			   subvol, dstdir);
+	else
+		pr_verbose(LOG_DEFAULT,
+			   "Create snapshot of '%s' in '%s'\n",
+			   subvol, dstdir);
+
 out:
-	close_file_or_dir(fddst, dirstream1);
-	close_file_or_dir(fd, dirstream2);
-	free(inherit);
-	free(dupname);
-	free(dupdir);
+	free(dstdir);
+	btrfs_util_qgroup_inherit_destroy(inherit);
 
 	return retval;
 }
@@ -826,7 +768,6 @@ static int cmd_subvolume_get_default(const struct cmd_struct *cmd, int argc, cha
 	int fd = -1;
 	int ret = 1;
 	uint64_t default_id;
-	DIR *dirstream = NULL;
 	enum btrfs_util_error err;
 	struct btrfs_util_subvolume_info subvol;
 	struct format_ctx fctx;
@@ -837,11 +778,11 @@ static int cmd_subvolume_get_default(const struct cmd_struct *cmd, int argc, cha
 	if (check_argc_exact(argc - optind, 1))
 		return 1;
 
-	fd = btrfs_open_dir(argv[1], &dirstream, 1);
+	fd = btrfs_open_dir(argv[1]);
 	if (fd < 0)
 		return 1;
 
-	err = btrfs_util_get_default_subvolume_fd(fd, &default_id);
+	err = btrfs_util_subvolume_get_default_fd(fd, &default_id);
 	if (err) {
 		error_btrfs_util(err);
 		goto out;
@@ -861,13 +802,13 @@ static int cmd_subvolume_get_default(const struct cmd_struct *cmd, int argc, cha
 		goto out;
 	}
 
-	err = btrfs_util_subvolume_info_fd(fd, default_id, &subvol);
+	err = btrfs_util_subvolume_get_info_fd(fd, default_id, &subvol);
 	if (err) {
 		error_btrfs_util(err);
 		goto out;
 	}
 
-	err = btrfs_util_subvolume_path_fd(fd, default_id, &path);
+	err = btrfs_util_subvolume_get_path_fd(fd, default_id, &path);
 	if (err) {
 		error_btrfs_util(err);
 		goto out;
@@ -889,7 +830,7 @@ static int cmd_subvolume_get_default(const struct cmd_struct *cmd, int argc, cha
 
 	ret = 0;
 out:
-	close_file_or_dir(fd, dirstream);
+	close(fd);
 	return ret;
 }
 #if EXPERIMENTAL
@@ -936,7 +877,7 @@ static int cmd_subvolume_set_default(const struct cmd_struct *cmd, int argc, cha
 		path = argv[optind + 1];
 	}
 
-	err = btrfs_util_set_default_subvolume(path, objectid);
+	err = btrfs_util_subvolume_set_default(path, objectid);
 	if (err) {
 		error_btrfs_util(err);
 		return 1;
@@ -961,9 +902,8 @@ static u64 find_root_gen(int fd)
 {
 	struct btrfs_ioctl_ino_lookup_args ino_args;
 	int ret;
-	struct btrfs_ioctl_search_args args;
-	struct btrfs_ioctl_search_key *sk = &args.key;
-	struct btrfs_ioctl_search_header sh;
+	struct btrfs_tree_search_args args;
+	struct btrfs_ioctl_search_key *sk;
 	unsigned long off = 0;
 	u64 max_found = 0;
 	int i;
@@ -979,7 +919,7 @@ static u64 find_root_gen(int fd)
 	}
 
 	memset(&args, 0, sizeof(args));
-
+	sk = btrfs_tree_search_sk(&args);
 	sk->tree_id = BTRFS_ROOT_TREE_OBJECTID;
 
 	/*
@@ -988,15 +928,15 @@ static u64 find_root_gen(int fd)
 	 * them.
 	 */
 	sk->min_objectid = ino_args.treeid;
+	sk->min_type = BTRFS_ROOT_ITEM_KEY;
 	sk->max_objectid = ino_args.treeid;
 	sk->max_type = BTRFS_ROOT_ITEM_KEY;
-	sk->min_type = BTRFS_ROOT_ITEM_KEY;
 	sk->max_offset = (u64)-1;
 	sk->max_transid = (u64)-1;
 	sk->nr_items = 4096;
 
 	while (1) {
-		ret = ioctl(fd, BTRFS_IOC_TREE_SEARCH, &args);
+		ret = btrfs_tree_search_ioctl(fd, &args);
 		if (ret < 0) {
 			error("can't perform the search: %m");
 			return 0;
@@ -1008,10 +948,11 @@ static u64 find_root_gen(int fd)
 		off = 0;
 		for (i = 0; i < sk->nr_items; i++) {
 			struct btrfs_root_item *item;
+			struct btrfs_ioctl_search_header sh;
 
-			memcpy(&sh, args.buf + off, sizeof(sh));
+			memcpy(&sh, btrfs_tree_search_data(&args, off), sizeof(sh));
 			off += sizeof(sh);
-			item = (struct btrfs_root_item *)(args.buf + off);
+			item = btrfs_tree_search_data(&args, off);
 			off += sh.len;
 
 			sk->min_objectid = sh.objectid;
@@ -1069,7 +1010,7 @@ static char *__ino_resolve(int fd, u64 dirid)
 		 */
 		full = strdup(args.name);
 		if (!full) {
-			error_msg(ERROR_MSG_MEMORY, NULL);
+			error_mem(NULL);
 			return ERR_PTR(-ENOMEM);
 		}
 	} else {
@@ -1112,14 +1053,13 @@ static char *ino_resolve(int fd, u64 ino, u64 *cache_dirid, char **cache_name)
 	char *name;
 	char *full;
 	int ret;
-	struct btrfs_ioctl_search_args args;
-	struct btrfs_ioctl_search_key *sk = &args.key;
-	struct btrfs_ioctl_search_header *sh;
-	unsigned long off = 0;
+	struct btrfs_tree_search_args args;
+	struct btrfs_ioctl_search_key *sk;
+	struct btrfs_ioctl_search_header sh;
 	int namelen;
 
 	memset(&args, 0, sizeof(args));
-
+	sk = btrfs_tree_search_sk(&args);
 	sk->tree_id = 0;
 
 	/*
@@ -1134,7 +1074,7 @@ static char *ino_resolve(int fd, u64 ino, u64 *cache_dirid, char **cache_name)
 	sk->max_transid = (u64)-1;
 	sk->nr_items = 1;
 
-	ret = ioctl(fd, BTRFS_IOC_TREE_SEARCH, &args);
+	ret = btrfs_tree_search_ioctl(fd, &args);
 	if (ret < 0) {
 		error("can't perform the search: %m");
 		return NULL;
@@ -1143,18 +1083,21 @@ static char *ino_resolve(int fd, u64 ino, u64 *cache_dirid, char **cache_name)
 	if (sk->nr_items == 0)
 		return NULL;
 
-	off = 0;
-	sh = (struct btrfs_ioctl_search_header *)(args.buf + off);
-
-	if (btrfs_search_header_type(sh) == BTRFS_INODE_REF_KEY) {
+	memcpy(&sh, btrfs_tree_search_data(&args, 0), sizeof(sh));
+	if (sh.type == BTRFS_INODE_REF_KEY) {
 		struct btrfs_inode_ref *ref;
-		dirid = btrfs_search_header_offset(sh);
 
-		ref = (struct btrfs_inode_ref *)(sh + 1);
+		dirid = sh.offset;
+
+		ref = btrfs_tree_search_data(&args, sizeof(sh));
 		namelen = btrfs_stack_inode_ref_name_len(ref);
 
 		name = (char *)(ref + 1);
 		name = strndup(name, namelen);
+		if (!name) {
+			error_mem(NULL);
+			return NULL;
+		}
 
 		/* use our cached value */
 		if (dirid == *cache_dirid && *cache_name) {
@@ -1260,9 +1203,8 @@ static int print_one_extent(int fd, struct btrfs_ioctl_search_header *sh,
 static int btrfs_list_find_updated_files(int fd, u64 root_id, u64 oldest_gen)
 {
 	int ret;
-	struct btrfs_ioctl_search_args args;
-	struct btrfs_ioctl_search_key *sk = &args.key;
-	struct btrfs_ioctl_search_header sh;
+	struct btrfs_tree_search_args args;
+	struct btrfs_ioctl_search_key *sk;
 	struct btrfs_file_extent_item *item;
 	unsigned long off = 0;
 	u64 found_gen;
@@ -1276,7 +1218,7 @@ static int btrfs_list_find_updated_files(int fd, u64 root_id, u64 oldest_gen)
 
 	memset(&backup, 0, sizeof(backup));
 	memset(&args, 0, sizeof(args));
-
+	sk = btrfs_tree_search_sk(&args);
 	sk->tree_id = root_id;
 
 	/*
@@ -1293,7 +1235,7 @@ static int btrfs_list_find_updated_files(int fd, u64 root_id, u64 oldest_gen)
 
 	max_found = find_root_gen(fd);
 	while(1) {
-		ret = ioctl(fd, BTRFS_IOC_TREE_SEARCH, &args);
+		ret = btrfs_tree_search_ioctl(fd, &args);
 		if (ret < 0) {
 			error("can't perform the search: %m");
 			break;
@@ -1309,7 +1251,9 @@ static int btrfs_list_find_updated_files(int fd, u64 root_id, u64 oldest_gen)
 		 * read the root_ref item it contains
 		 */
 		for (i = 0; i < sk->nr_items; i++) {
-			memcpy(&sh, args.buf + off, sizeof(sh));
+			struct btrfs_ioctl_search_header sh;
+
+			memcpy(&sh, btrfs_tree_search_data(&args, off), sizeof(sh));
 			off += sizeof(sh);
 
 			/*
@@ -1319,8 +1263,8 @@ static int btrfs_list_find_updated_files(int fd, u64 root_id, u64 oldest_gen)
 			if (sh.len == 0)
 				item = &backup;
 			else
-				item = (struct btrfs_file_extent_item *)(args.buf +
-								 off);
+				item = btrfs_tree_search_data(&args, off);
+
 			found_gen = btrfs_stack_file_extent_generation(item);
 			if (sh.type == BTRFS_EXTENT_DATA_KEY &&
 			    found_gen >= oldest_gen) {
@@ -1335,8 +1279,8 @@ static int btrfs_list_find_updated_files(int fd, u64 root_id, u64 oldest_gen)
 			 * next search doesn't repeat this root
 			 */
 			sk->min_objectid = sh.objectid;
-			sk->min_offset = sh.offset;
 			sk->min_type = sh.type;
+			sk->min_offset = sh.offset;
 		}
 		sk->nr_items = 4096;
 		if (sk->min_offset < (u64)-1)
@@ -1360,7 +1304,6 @@ static int cmd_subvolume_find_new(const struct cmd_struct *cmd, int argc, char *
 	int ret;
 	char *subvol;
 	u64 last_gen;
-	DIR *dirstream = NULL;
 	enum btrfs_util_error err;
 
 	clean_args_no_options(cmd, argc, argv);
@@ -1371,25 +1314,25 @@ static int cmd_subvolume_find_new(const struct cmd_struct *cmd, int argc, char *
 	subvol = argv[optind];
 	last_gen = arg_strtou64(argv[optind + 1]);
 
-	err = btrfs_util_is_subvolume(subvol);
+	err = btrfs_util_subvolume_is_valid(subvol);
 	if (err) {
 		error_btrfs_util(err);
 		return 1;
 	}
 
-	fd = btrfs_open_dir(subvol, &dirstream, 1);
+	fd = btrfs_open_dir(subvol);
 	if (fd < 0)
 		return 1;
 
-	err = btrfs_util_sync_fd(fd);
+	err = btrfs_util_fs_sync_fd(fd);
 	if (err) {
 		error_btrfs_util(err);
-		close_file_or_dir(fd, dirstream);
+		close(fd);
 		return 1;
 	}
 
 	ret = btrfs_list_find_updated_files(fd, 0, last_gen);
-	close_file_or_dir(fd, dirstream);
+	close(fd);
 	return !!ret;
 }
 static DEFINE_SIMPLE_COMMAND(subvolume_find_new, "find-new");
@@ -1563,7 +1506,6 @@ static int cmd_subvolume_show(const struct cmd_struct *cmd, int argc, char **arg
 	char *fullpath = NULL;
 	int fd = -1;
 	int ret = 1;
-	DIR *dirstream1 = NULL;
 	int by_rootid = 0;
 	int by_uuid = 0;
 	u64 rootid_arg = 0;
@@ -1571,7 +1513,7 @@ static int cmd_subvolume_show(const struct cmd_struct *cmd, int argc, char **arg
 	struct btrfs_util_subvolume_iterator *iter;
 	struct btrfs_util_subvolume_info subvol;
 	char *subvol_path = NULL;
-	char *subvol_name = NULL;
+	const char *subvol_name = NULL;
 	enum btrfs_util_error err;
 	struct btrfs_qgroup_stats stats;
 	unsigned int unit_mode;
@@ -1621,34 +1563,31 @@ static int cmd_subvolume_show(const struct cmd_struct *cmd, int argc, char **arg
 		goto out;
 	}
 
-	fd = open_file_or_dir(fullpath, &dirstream1);
+	fd = btrfs_open_file_or_dir(fullpath);
 	if (fd < 0) {
-		error("can't access '%s'", fullpath);
+		ret = fd;
 		goto out;
 	}
 
 	if (by_uuid) {
-		err = btrfs_util_create_subvolume_iterator_fd(fd,
-							      BTRFS_FS_TREE_OBJECTID,
-							      0, &iter);
+		err = btrfs_util_subvolume_iter_create_fd(fd, BTRFS_FS_TREE_OBJECTID,
+							  0, &iter);
 		if (err) {
 			error_btrfs_util(err);
 			goto out;
 		}
 
 		for (;;) {
-			err = btrfs_util_subvolume_iterator_next_info(iter,
-								      &subvol_path,
-								      &subvol);
+			err = btrfs_util_subvolume_iter_next_info(iter, &subvol_path, &subvol);
 			if (err == BTRFS_UTIL_ERROR_STOP_ITERATION) {
 				uuid_unparse(uuid_arg, uuidparse);
 				error("can't find uuid '%s' on '%s'", uuidparse,
 				      fullpath);
-				btrfs_util_destroy_subvolume_iterator(iter);
+				btrfs_util_subvolume_iter_destroy(iter);
 				goto out;
 			} else if (err) {
 				error_btrfs_util(err);
-				btrfs_util_destroy_subvolume_iterator(iter);
+				btrfs_util_subvolume_iter_destroy(iter);
 				goto out;
 			}
 
@@ -1658,19 +1597,19 @@ static int cmd_subvolume_show(const struct cmd_struct *cmd, int argc, char **arg
 			free(subvol_path);
 			subvol_path = NULL;
 		}
-		btrfs_util_destroy_subvolume_iterator(iter);
+		btrfs_util_subvolume_iter_destroy(iter);
 	} else {
 		/*
 		 * If !by_rootid, rootid_arg = 0, which means find the
 		 * subvolume ID of the fd and use that.
 		 */
-		err = btrfs_util_subvolume_info_fd(fd, rootid_arg, &subvol);
+		err = btrfs_util_subvolume_get_info_fd(fd, rootid_arg, &subvol);
 		if (err) {
 			error_btrfs_util(err);
 			goto out;
 		}
 
-		err = btrfs_util_subvolume_path_fd(fd, subvol.id, &subvol_path);
+		err = btrfs_util_subvolume_get_path_fd(fd, subvol.id, &subvol_path);
 		if (err) {
 			error_btrfs_util(err);
 			goto out;
@@ -1683,7 +1622,7 @@ static int cmd_subvolume_show(const struct cmd_struct *cmd, int argc, char **arg
 		subvol_path = strdup("/");
 		subvol_name = "<FS_TREE>";
 	} else {
-		subvol_name = basename(subvol_path);
+		subvol_name = path_basename(subvol_path);
 	}
 
 	if (bconf.output_format == CMD_FORMAT_JSON) {
@@ -1700,20 +1639,18 @@ static int cmd_subvolume_show(const struct cmd_struct *cmd, int argc, char **arg
 	else
 		pr_verbose(LOG_DEFAULT, "\tSnapshot(s):\n");
 
-	err = btrfs_util_create_subvolume_iterator_fd(fd,
-						      BTRFS_FS_TREE_OBJECTID, 0,
-						      &iter);
+	err = btrfs_util_subvolume_iter_create_fd(fd, BTRFS_FS_TREE_OBJECTID, 0, &iter);
 
 	for (;;) {
 		struct btrfs_util_subvolume_info subvol2;
 		char *path;
 
-		err = btrfs_util_subvolume_iterator_next_info(iter, &path, &subvol2);
+		err = btrfs_util_subvolume_iter_next_info(iter, &path, &subvol2);
 		if (err == BTRFS_UTIL_ERROR_STOP_ITERATION) {
 			break;
 		} else if (err) {
 			error_btrfs_util(err);
-			btrfs_util_destroy_subvolume_iterator(iter);
+			btrfs_util_subvolume_iter_destroy(iter);
 			goto out2;
 		}
 
@@ -1730,7 +1667,7 @@ static int cmd_subvolume_show(const struct cmd_struct *cmd, int argc, char **arg
 	if (bconf.output_format == CMD_FORMAT_JSON)
 		fmt_print_end_group(&fctx, "snapshots");
 
-	btrfs_util_destroy_subvolume_iterator(iter);
+	btrfs_util_subvolume_iter_destroy(iter);
 
 	ret = btrfs_qgroup_query(fd, subvol.id, &stats);
 	if (ret == -ENOTTY) {
@@ -1759,7 +1696,7 @@ out2:
 
 out:
 	free(subvol_path);
-	close_file_or_dir(fd, dirstream1);
+	close(fd);
 	free(fullpath);
 	return !!ret;
 }
@@ -1786,7 +1723,6 @@ static int cmd_subvolume_sync(const struct cmd_struct *cmd, int argc, char **arg
 {
 	int fd = -1;
 	int ret = 1;
-	DIR *dirstream = NULL;
 	uint64_t *ids = NULL;
 	size_t id_count, i;
 	int sleep_interval = 1;
@@ -1816,7 +1752,7 @@ static int cmd_subvolume_sync(const struct cmd_struct *cmd, int argc, char **arg
 	if (check_argc_min(argc - optind, 1))
 		return 1;
 
-	fd = btrfs_open_dir(argv[optind], &dirstream, 1);
+	fd = btrfs_open_dir(argv[optind]);
 	if (fd < 0) {
 		ret = 1;
 		goto out;
@@ -1825,7 +1761,7 @@ static int cmd_subvolume_sync(const struct cmd_struct *cmd, int argc, char **arg
 
 	id_count = argc - optind;
 	if (!id_count) {
-		err = btrfs_util_deleted_subvolumes_fd(fd, &ids, &id_count);
+		err = btrfs_util_subvolume_list_deleted_fd(fd, &ids, &id_count);
 		if (err) {
 			error_btrfs_util(err);
 			ret = 1;
@@ -1838,7 +1774,7 @@ static int cmd_subvolume_sync(const struct cmd_struct *cmd, int argc, char **arg
 	} else {
 		ids = malloc(id_count * sizeof(uint64_t));
 		if (!ids) {
-			error_msg(ERROR_MSG_MEMORY, NULL);
+			error_mem(NULL);
 			ret = 1;
 			goto out;
 		}
@@ -1869,7 +1805,7 @@ static int cmd_subvolume_sync(const struct cmd_struct *cmd, int argc, char **arg
 
 out:
 	free(ids);
-	close_file_or_dir(fd, dirstream);
+	close(fd);
 
 	return !!ret;
 }
