@@ -51,6 +51,7 @@
 #include "common/utils.h"
 #include "common/defs.h"
 #include "common/open-utils.h"
+#include "common/string-utils.h"
 #include "common/units.h"
 
 static int btrfs_scan_done = 0;
@@ -147,7 +148,7 @@ int btrfs_add_to_fsid(struct btrfs_trans_handle *trans,
 	if (!device)
 		return -ENOMEM;
 
-	buf = calloc(1, sectorsize);
+	buf = calloc(1, BTRFS_SUPER_INFO_SIZE);
 	if (!buf) {
 		ret = -ENOMEM;
 		goto out;
@@ -237,7 +238,7 @@ int btrfs_register_one_device(const char *fname)
 		return -errno;
 	}
 	memset(&args, 0, sizeof(args));
-	strncpy_null(args.name, fname);
+	strncpy_null(args.name, fname, sizeof(args.name));
 	ret = ioctl(fd, BTRFS_IOC_SCAN_DEV, &args);
 	if (ret < 0) {
 		error("device scan failed on '%s': %m", fname);
@@ -293,8 +294,7 @@ int btrfs_device_already_in_root(struct btrfs_root *root, int fd,
 			btrfs_super_magic(&disk_super) != BTRFS_MAGIC_TEMPORARY)
 		goto out;
 
-	if (!memcmp(disk_super.fsid, root->fs_info->super_copy->fsid,
-		    BTRFS_FSID_SIZE))
+	if (memcmp(disk_super.fsid, root->fs_info->super_copy->fsid, BTRFS_FSID_SIZE) == 0)
 		ret = 1;
 out:
 	return ret;
@@ -468,7 +468,7 @@ int btrfs_scan_devices(int verbose)
 		if (!dev)
 			continue;
 		/* if we are here its definitely a btrfs disk*/
-		strncpy_null(path, blkid_dev_devname(dev));
+		strncpy_null(path, blkid_dev_devname(dev), sizeof(path));
 
 		if (stat(path, &dev_stat) < 0)
 			continue;
