@@ -948,8 +948,7 @@ static int make_convert_data_block_groups(struct btrfs_trans_handle *trans,
 			u64 cur_backup = cur;
 
 			len = min(max_chunk_size,
-				  round_up(cache->start + cache->size,
-					   BTRFS_STRIPE_LEN) - cur);
+				  cache->start + cache->size - cur);
 			ret = btrfs_alloc_data_chunk(trans, fs_info, &cur_backup, len);
 			if (ret < 0)
 				break;
@@ -1250,7 +1249,7 @@ static int do_convert(const char *devname, u32 convert_flags, u32 nodesize,
 	uuid_unparse(cctx.fs_uuid, fsid_str);
 	printf("  UUID:           %s\n", fsid_str);
 	printf("Target filesystem:\n");
-	printf("  Label:          %s\n", fslabel);
+	printf("  Label:          %s\n", (convert_flags & CONVERT_FLAG_COPY_LABEL) ? cctx.label : fslabel);
 	printf("  Blocksize:      %u\n", blocksize);
 	printf("  Nodesize:       %u\n", nodesize);
 	printf("  UUID:           %s\n", mkfs_cfg.fs_uuid);
