@@ -14,8 +14,6 @@
 */
 
 #include <stdint.h>
-#include <string.h>
-
 #include "blake2.h"
 #include "blake2-impl.h"
 #include "blake2-config.h"
@@ -30,6 +28,7 @@
 #include <x86intrin.h>
 #endif
 
+#include "blake2b-load-sse2.h"
 #include "blake2b-round.h"
 
 static const uint64_t blake2b_IV[8] =
