@@ -15,9 +15,12 @@
  */
 
 #include "kerncompat.h"
+#include <stdbool.h>
 #include <stdlib.h>
 #include <string.h>
 #include "kernel-lib/rbtree.h"
+#include "kernel-shared/accessors.h"
+#include "kernel-shared/uapi/btrfs_tree.h"
 #include "kernel-shared/ctree.h"
 #include "kernel-shared/extent_io.h"
 #include "common/internal.h"
@@ -260,7 +263,7 @@ static char *find_collision(struct rb_root *name_tree, char *name,
 
 	val = malloc(sizeof(struct name));
 	if (!val) {
-		error_msg(ERROR_MSG_MEMORY, "sanitize name");
+		error_mem("sanitize name");
 		free(name);
 		return NULL;
 	}
@@ -271,7 +274,7 @@ static char *find_collision(struct rb_root *name_tree, char *name,
 	val->len = name_len;
 	val->sub = malloc(name_len);
 	if (!val->sub) {
-		error_msg(ERROR_MSG_MEMORY, "sanitize name");
+		error_mem("sanitize name");
 		free(val);
 		free(name);
 		return NULL;
@@ -340,7 +343,7 @@ static void sanitize_dir_item(enum sanitize_mode sanitize,
 		if (sanitize == SANITIZE_COLLISIONS) {
 			buf = malloc(name_len);
 			if (!buf) {
-				error_msg(ERROR_MSG_MEMORY, "sanitize name");
+				error_mem("sanitize name");
 				return;
 			}
 			read_extent_buffer(eb, buf, name_ptr, name_len);
@@ -349,7 +352,7 @@ static void sanitize_dir_item(enum sanitize_mode sanitize,
 			garbage = generate_garbage(name_len);
 		}
 		if (!garbage) {
-			error_msg(ERROR_MSG_MEMORY, "sanitize name");
+			error_mem("sanitize name");
 			return;
 		}
 		write_extent_buffer(eb, garbage, name_ptr, name_len);
@@ -395,7 +398,7 @@ static void sanitize_inode_ref(enum sanitize_mode sanitize,
 		if (sanitize == SANITIZE_COLLISIONS) {
 			buf = malloc(len);
 			if (!buf) {
-				error_msg(ERROR_MSG_MEMORY, "sanitize name");
+				error_mem("sanitize name");
 				return;
 			}
 			read_extent_buffer(eb, buf, name_ptr, len);
@@ -405,7 +408,7 @@ static void sanitize_inode_ref(enum sanitize_mode sanitize,
 		}
 
 		if (!garbage) {
-			error_msg(ERROR_MSG_MEMORY, "sanitize name");
+			error_mem("sanitize name");
 			return;
 		}
 		write_extent_buffer(eb, garbage, name_ptr, len);
@@ -446,10 +449,12 @@ void sanitize_name(enum sanitize_mode sanitize, struct rb_root *name_tree,
 		int slot)
 {
 	struct extent_buffer *eb;
+	u32 item_ptr_off = btrfs_item_ptr_offset(src, slot);
+	u32 item_ptr_size = btrfs_item_size(src, slot);
 
 	eb = alloc_dummy_eb(src->start, src->len);
 	if (!eb) {
-		error_msg(ERROR_MSG_MEMORY, "sanitize name");
+		error_mem("sanitize name");
 		return;
 	}
 
@@ -473,7 +478,11 @@ void sanitize_name(enum sanitize_mode sanitize, struct rb_root *name_tree,
 		break;
 	}
 
-	memcpy(dst, eb->data, eb->len);
+	/*
+	 * Only overwrite the sanitized part, or we can overwrite previous
+	 * sanitized value with the old values from @src.
+	 */
+	memcpy(dst + item_ptr_off, eb->data + item_ptr_off, item_ptr_size);
 	free(eb);
 }
 
