@@ -1,6 +1,8 @@
 #ifndef __PROGS_TRACE_H__
 #define __PROGS_TRACE_H__
 
+#include "kerncompat.h"
+
 struct btrfs_work;
 struct btrfs_fs_info;
 struct extent_state;
@@ -57,4 +59,10 @@ static inline void trace_btrfs_convert_extent_bit(struct extent_io_tree *tree,
 {
 }
 
+static inline void trace_btrfs_cow_block(struct btrfs_root *root,
+					 struct extent_buffer *buf,
+					 struct extent_buffer *cow)
+{
+}
+
 #endif /* __PROGS_TRACE_H__ */
