@@ -20,7 +20,6 @@
 #include <stdbool.h>
 #include <stdint.h>
 #include <string.h>
-#include <errno.h>
 #include "kernel-lib/list.h"
 #include "kernel-shared/accessors.h"
 #include "kernel-shared/extent_io.h"
@@ -650,7 +649,8 @@ static int __add_inline_refs(struct btrfs_fs_info *fs_info,
 			break;
 		}
 		default:
-			WARN_ON(1);
+			error("invalid backref type: %u", type);
+			ret = -EUCLEAN;
 		}
 		if (ret)
 			return ret;
