@@ -1,16 +1,17 @@
 #!/bin/bash
 # Check if btrfs-convert can copy common inode flags like SYNC/IMMUTABLE
 
-source "$TOP/tests/common"
-source "$TOP/tests/common.convert"
+source "$TEST_TOP/common" || exit
+source "$TEST_TOP/common.convert" || exit
 
-setup_root_helper
-prepare_test_dev 512M
 check_prereq btrfs-convert
 check_global_prereq mke2fs
 check_global_prereq lsattr
 check_global_prereq chattr
 
+setup_root_helper
+prepare_test_dev
+
 fail=0
 default_mke2fs="mke2fs -t ext4 -b 4096"
 convert_test_preamble '' 'common inode flags test' 16k "$default_mke2fs"
