@@ -1,20 +1,21 @@
 #!/bin/bash
 # Check if btrfs-convert can copy common inode flags like SYNC/IMMUTABLE
 
-source "$TOP/tests/common"
-source "$TOP/tests/common.convert"
+source "$TEST_TOP/common" || exit
+source "$TEST_TOP/common.convert" || exit
 
 if ! check_kernel_support_reiserfs >/dev/null; then
 	_not_run "no reiserfs support"
 fi
 
-setup_root_helper
-prepare_test_dev 512M
 check_prereq btrfs-convert
 check_global_prereq mkreiserfs
 check_global_prereq chattr
 check_global_prereq lsattr
 
+setup_root_helper
+prepare_test_dev
+
 fail=0
 default_mkfs="mkreiserfs -b 4096"
 convert_test_preamble '' 'common inode flags test' 16k "$default_mkfs"
