@@ -2,10 +2,11 @@
 
 # iterate over all fuzzed images and run 'btrfs-image'
 
-source "$TEST_TOP/common"
+source "$TEST_TOP/common" || exit
+
+check_prereq btrfs-image
 
 setup_root_helper
-check_prereq btrfs-image
 
 # redefine the one provided by common
 check_image() {
