@@ -2,7 +2,7 @@
 # Test that receive determines the correct mount point path when there is
 # another mount point that matches the destination's path as a prefix.
 
-source "$TEST_TOP/common"
+source "$TEST_TOP/common" || exit
 
 # fix reverted in v4.20.2 due to reported breakage, the bug fixed by
 # "Btrfs-progs: fix mount point detection due to partial prefix match" is still
@@ -23,9 +23,11 @@ loop2=$(run_check_stdout $SUDO_HELPER losetup --find --show dev2)
 run_check $SUDO_HELPER "$TOP/mkfs.btrfs" -f "$loop1"
 run_check $SUDO_HELPER "$TOP/mkfs.btrfs" -f "$loop2"
 
+cond_wait_for_loopdevs
 run_check $SUDO_HELPER mount "$loop1" "$TEST_MNT"
 run_check $SUDO_HELPER mkdir "$TEST_MNT/ddis"
 run_check $SUDO_HELPER mkdir "$TEST_MNT/ddis-not-a-mount"
+cond_wait_for_loopdevs
 run_check $SUDO_HELPER mount "$loop2" "$TEST_MNT/ddis"
 
 echo "some data" | $SUDO_HELPER tee "$TEST_MNT/ddis/file" > /dev/null
